
#    LUCI is an Universal Configuration Interface
#    Copyright (C) 1999-2001 Jrme SAUTRET <Jerome.Sautret@netcourrier.com>
#                                                                
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or   
#    (at your option) any later version.                              
#                                      
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of 
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
#    GNU General Public License for more details.                 
#                                               
#    You should have received a copy of the GNU General Public License
#    along with this program; see the file COPYING.  If not, write to 
#    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#    Boston, MA 02111-1307, USA.

import string
import types
import re
import os
import errno

debug=0

comments={ 'shell':'#', 'shell_env':'#', 'keywords':'#', 'set':'#'}
patterns={}
patterns['shell']=r'\s*(?P<comment>#?)\s*(?P<variable>\b\w+\b)=(?P<value>\S*(?:\s*[^\s#]+)*)(?:\s*)|(?:\s+#.*)$'
patterns['shell_env']=r'\s*(?P<comment>#?)\s*export\s+(?P<variable>\b\w+\b)=(?P<value>\S*(?:\s*[^\s#]+)*)(?:\s*)|(?:\s+#.*)$'
patterns['keywords']=r'\s*(?P<comment>#?)\s*(?P<variable>\b\w+\b)\s+(?P<value>\S*(?:\s*[^\s#]+)*)(?:\s*)|(?:\s+#.*)$'
patterns['set']=r'\s*(?P<comment>#?)\s*set\s+(?P<variable>\b[\w-]+\b)\s+(?P<value>\S*(?:\s*[^\s#]+)*)(?:\s*)|(?:\s+#.*)$'
new_entries={ 'shell'    :'<comment><name>=<value>\n',
              'shell_env':'<comment>export <name>=<value>\n',
              'keywords':'<comment><name> <value>\n',
              'set':'<comment>set <name> <value>\n'}


def init_params(params):
    type=params['config_file_type']
    
    if not params.has_key('comment') and comments.has_key(type):
        params['comment']=comments[type]
    if not params.has_key('pattern'):
        params['pattern']=re.compile(patterns[type])
    else:
        print params['pattern']
        params['pattern']=re.compile(params['pattern'])
    if not params.has_key('new_entry'):
        params['new_entry']=new_entries[type]
    

def set_value_entry(entry, value):
    entry['value']=value
    if entry.has_key('value_type') and entry['value_type']=='list':
	for i in entry['list_values']:
	    if type(i)==types.TupleType and i[1]==entry['value']:
		entry['value']=i[0]

def match_read(s, parameters, entries):
    
    m=parameters['pattern'].match(s)
    try:
	if m:
	    for i in entries.keys():
		if m.group('variable')==entries[i]['config_name']:
		    if m.groupdict().has_key('comment'):
			if not m.group('comment'):   
			    entries[i]['_active']=1    # entry not commented
			    set_value_entry(entries[i],m.group('value'))
			elif not entries[i].has_key('_active') or \
			     entries[i]['_active']==0:    
			    entries[i]['_active']=0    # entry commented
			    set_value_entry(entries[i],m.group('value'))
		    else:
			set_value_entry(entries[i],m.group('value'))
			
    except KeyError, p:
	print "Error: cannot find parameter '"+p.args[0]+"' for entry '"+\
	      i+"'"
	print "in description file "+parameters['desc_file_name']+"."
	raise SystemExit, 7
	

def read_file(config, entries):
    try:
	file=config['config_file_name']
        if string.strip(file)[0:2]=='~/':
            file=string.replace(file,'~',os.environ['HOME'],1)
            config['config_file_name']=file
	f=open(file, 'r')
	line=f.readline()
	while line != '':
	    match_read(line, config, entries)
	    line=f.readline()
	f.close()
    except KeyError, p:
	print "Error: cannot find parameter '"+p.args[0]+"'"
	print "in description file "+config['desc_file_name']+"."
	raise SystemExit, 7
    except IOError, m:
        if errno.errorcode[m.errno] == 'ENOENT' :
            return "NoConfigFile"
	print "Error in reading description file:"
	print m
	raise SystemExit, 6

def match_write(s, parameters, entries):
    
    p=parameters['pattern']
    m=p.match(s)
    if m:
	for i in entries.keys():
	    if m.group('variable')==entries[i]['config_name']:
                if (entries[i].has_key('presence') and \
                    entries[i]['presence']=='optional'):
                    if entries[i]['_active']:
                        c=''
                    else:
                        c=parameters['comment']
                    if debug:
                        print m.group('variable'),c
                    s=s[0:m.start('comment')]+c+\
                       s[m.end('comment'):m.start('value')]+\
                       entries[i]['value']+s[m.end('value'):]
                else:
                    s=s[0:m.start('value')]+entries[i]['value']+s[m.end('value'):]
		entries[i]['_written']=1
    return s

def create_entry(config, entry):
    new_entry=config['new_entry']
    new_entry=string.replace(new_entry,'<name>',entry['config_name'])
    new_entry=string.replace(new_entry,'<value>',entry['value'])
    c=''
    if ( entry.has_key('presence') and \
         entry['presence']=='optional' and \
         not entry['_active']):
        c=config['comment']
    if config.has_key('comment'):
        new_entry=string.replace(new_entry,'<comment>',c)

    return new_entry

def write_file(config, entries):
    try:

        fileout=config['config_file_name']
        filein=fileout+'.bak'
        try:
            os.rename(fileout,filein)
        except OSError, m:  # if config file doesn't exists, create it empty
            if errno.errorcode[m.errno] == 'ENOENT' :
                fin=open(filein, 'w')
                fin.close()
            else:
                raise OSError, m
        fin=open(filein, 'r')
        fout=open(fileout, 'w')
        line=fin.readline()
        while line != '':
            fout.write(match_write(line, config, entries))
            line=fin.readline()
        for entry in entries.keys():
            if not entries[entry].has_key('_written') or not entries[entry]['_written']:
                if debug:
                    print entry
                fout.write(create_entry(config, entries[entry]))
        fin.close()
        fout.close()

    except KeyError, k:
        print "Error, parameter '"+k.args[0]+"' is missing in description file."
        raise SystemExit, 10
    except IndexError, g:
        print "Error, "+g.args[0]+" in regular expression define by pattern."
        raise SystemExit, 10
    except OSError, m:
        if m.filename:
            file = m.filename
        else:
            file = fileout
        print "Error on file",file,"while saving configuration:"
        print m
        raise SystemExit, 9
        
